package Caching;
import java.util.*;
import HTTPClient.*;
import java.lang.reflect.*;

/**
 * This class is used to create and manage a buffer of information about
 * the pages which we have cached <br>
 * There is no constructor for this class because you shoudn't ever need to
 * make a NEW instance of this class. This class is designed to make an instance
 * of itself only once when using the getInstance command (see the method for more info)
 * <br>
 * <br>
 * Whenever you need to use this object you need to make an instance of it as follows: <br>
 * CacheBuffer cb = CacheBuffer.getInstance(); <br>
 */
import StartAllServers.Debug;
public class CacheBuffer{
    
    private static CacheBuffer myself;

    public CacheTreeNode Tree = new CacheTreeNode();

    /**
     * Make an instance of the Buffer. <br>
     */
    private CacheBuffer() {
	Logging log = Logging.getInstance();
	Hashtable temp = new Hashtable();
	try {
	    temp = log.readMasterFile();
	} catch (Exception e) {
	    System.err.println(e.getMessage());
	}
	if (temp != null)
	    urls = (Hashtable) temp.clone();
	else 
	    urls = new Hashtable();

    }

    /**
     * Get an instance of this buffer. <br>
     * Since we want this buffer to be global everyone needs to create an instance of this
     * object before making any modifications to any part of it. If the object doesn't
     * exist it will be made anew and a reference to the new instance will be returned
     */
    public static synchronized CacheBuffer getInstance() {
	if (myself == null) {
	    myself = new CacheBuffer();
	}
	return myself;
    }

    //Hash of URLs and their attributes;
    Hashtable urls;
    
   
    /**
     * Print out the contents of the buffer
     */

    public void printBuffer() {

	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("%%%%%%%%%%%%The Buffer contains:");
 	}

	 for (Enumeration e = urls.elements() ; e.hasMoreElements() ;) {
	     BufferEntry entry = (BufferEntry) e.nextElement();

	     if (Debug.DEBUG_Caching == true) { 
		 System.out.println(entry); 
		 
	     }
	 }
    }
    


    /**
     * Add a URL to the buffer. <br>
     * This procedure basically takes a URL and an object of type HTTPResponse
     * and extract the headers from the HTTP response. The headers it currently
     * extract are: <br>
     * (1) Date of Expiration <br>
     * (2) Date when the URL was obtained <br>
     * (3) Date when the URL was last-modified on the server <br>
     */
    public void addURLToBuffer(String URL, HTTPResponse rsp ) {
	BufferEntry entry = new BufferEntry();

	try {
	    entry.setExpired(rsp.getHeaderAsDate("Expires"));
	    
	    if (Debug.DEBUG_Caching == true) { 
		System.out.println("Expires on:" + entry.expires_on.toString()); 
	    } 
	    
	} catch (Exception e) {}
	
	try {
	    entry.setModified(rsp.getHeaderAsDate("Last-Modified"));
	    
	    if (Debug.DEBUG_Caching == true) { 
		
		System.out.println("Last Modified on: " + entry.modified_on.toString()); 
		
	    }
	} catch (Exception e) {}
	
	try {
	    //entry.setRetrieved(rsp.getHeaderAsDate("Date"));
	    entry.setRetrieved(new Date());
	    if (Debug.DEBUG_Caching == true) { 
		
		System.out.println("Retrieved on: " + entry.getRetrieved()); 
		
	    }
	} catch (Exception e) {}

	entry.setUrl(URL);
	urls.put(URL, entry);
	Logging log = Logging.getInstance();
	log.addAddMessToLogFile(entry);

    }
    
    /**
     * Replace the contents of the Buffer
     */
    public boolean replaceBuffer(Hashtable new_buffer) {
	this.urls = (Hashtable) new_buffer.clone();
	return true;
    }    


    /**
     * Give the contents of the buffer
     */
    public Hashtable giveBuffer() {
	Hashtable return_me = (Hashtable) urls.clone();
	return return_me;
    }

    /**
     * See if an entry is in the cache buffer
     */
    public boolean isInBuffer(BufferEntry entry) {
	return urls.contains(entry);
    }

    /**
     * Update an entry in the cache Buffer
     */
    public boolean updateBuffer(String url_to_update, BufferEntry new_entry) {

	BufferEntry entry = getBufferEntry(url_to_update);   

	if (entry == null) 
	    return false;
	
	entry = (BufferEntry) new_entry.clone();
	Logging log = Logging.getInstance();
	log.addUpdateMessToLogFile(entry);
	return true;
    }
    

    /**
     * Update an entry in the cache buffer
     */
    public boolean updateBuffer(BufferEntry old, BufferEntry new_entry) {
	BufferEntry entry = getBufferEntry(old.getUrl());
	if (entry == null) 
	    return false;
	
	entry = (BufferEntry) new_entry.clone();
	Logging log = Logging.getInstance();
	log.addUpdateMessToLogFile(entry);
	return true;
    }


    /**
     * Add an entry to the cache buffer
     */
    public boolean addToBuffer(BufferEntry entry) {
	//if it already exists then update the entry - can't have duplicate urls
	if (getBufferEntry(entry.getUrl()) != null) {
	    updateBuffer(entry.getUrl(), entry);
	    return true;
	}
	//add to the buffer
	else {
	    urls.put(entry.getUrl(), entry);
	    Logging log = Logging.getInstance();
	    log.addAddMessToLogFile(entry);
	    return true;
	}
    }

    /**
     * Delete an entry from the cache buffer
     */
    public boolean deleteBufferEntry(BufferEntry entry) {
	Object test = urls.remove(entry.url);
	if (test == null)
	    return false;
	else {
	    Logging log = Logging.getInstance();
	    log.addDeleteMessToLogFile(entry);
	    return true;
	}
    }

    /** 
     * Get the buffer structure associated with a URL. <br>
     * returns null if the URL is not present in the Buffer
     */
    public BufferEntry getBufferEntry(String URL) {
	return (BufferEntry) urls.get(URL);
    }

    /**
     * Delete the URL from the Cache Buffer. <br>
     * returns true if the deletion was successfull, false otherwise
     */
    public boolean deleteBufferEntry(String URL) {
	BufferEntry test = (BufferEntry) urls.remove(URL);
	if (test == null) 
	    return false;
	else {
	    Logging log = Logging.getInstance();
	    log.addDeleteMessToLogFile(test);
	    return true;
	}
    }
     
    /**
     * Get the expired field of a certain URL. <br>
     * returns the Date Object (@see java.util.Date)
     */
    public Date getExpiration(String URL) {
	if (getBufferEntry(URL) == null) 
	    return null;
	return getBufferEntry(URL).getExpired();
    }
    
    /**
     * Get the date retrieved field of a certain URL. <br>
     * returns the Date Object (@see java.util.Date)
     */
    public Date getRetrieved(String URL) {
	if (getBufferEntry(URL) == null) 
	    return null;
	return getBufferEntry(URL).getRetrieved();
    }	
    
    /**
     * Get the date last modified field of a certain URL. <br>
     * returns the Date Object (@see java.util.Date)
     */
    public Date getLastModified(String URL) {
	URL = URL.trim();
	if (getBufferEntry(URL) == null) 
	    return null;
	return getBufferEntry(URL).getModified();
    }

    
}
    

	    
