package Caching;
import java.lang.*;
import java.io.*;
import java.util.*; 
import java.text.*;



import StartAllServers.Debug;
public class Logging {

    String token_separator = "\"";
    String master = ".cacheIndexF";
    String temp = ".temp";
    String log = ".log";
    FileOutputStream flog; //this will be used to write to the log
    boolean merged;  //this indicates whether the file has been merged

    Hashtable masterhash = new Hashtable();
    Hashtable loghash = new Hashtable();
    public boolean ran_once = false;

    private static Logging myself;

    /**
     * Make an instance of the Log. <br>
     */
    private Logging() {
	boolean pass;

	if (new File(log).exists()) {
	    //something happened before creating temp so redo merge
	    new File(temp).delete(); //delete the temp file if it exists
	    merge();
	}

	else if (new File(temp).exists()) {
	    //since the previous case captures the case when the log file exists then it
	    //can't exist in this case so it was deleted therefore the temp file was created
	    //successfully so all we need to do is to copy the contents of the tempfile to 
	    //the main file 
	    pass = mergeTempToReal();
	    if (pass)
		reCreateLog();
	}
	else {
	    //if the log nor the temp file exist then before the system was shut off the merge
	    //went ok so just create the log
	    reCreateLog();
	    try {
		if (!new File(master).exists()) 
		    new File(master).createNewFile();
	    } catch (IOException e) {

		if (Debug.DEBUG_Caching == true) { 
		    System.out.println("Can't create index file:" + e.getMessage()); 
		} 

		System.exit(1);
	    }
	}
	cleanUp();

    }

	

    /**
     * Get an instance of this Log. <br>
     * Since we want this buffer to be global everyone needs to create an instance of this
     * object before making any modifications to any part of it. If the object doesn't
     * exist it will be made anew and a reference to the new instance will be returned
     */
    public static synchronized Logging getInstance() {
	if (myself == null) {
	    myself = new Logging();
	}
	return myself;
    }

   
    /**
     * Merge the contents of the CacheIndexF and the log
     */

    public boolean merge() {

	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("Merging log"); 
 	
 	}
	try {
	    boolean pass = false;

	    readMasterFile(); 
	    pass = true;
	    
	    if (pass)
		pass = actChangesInLog();
	    
	    if (pass) 
		pass = writeTempIndex();

	    if (pass)
		pass = mergeTempToReal();
	  	   
	    if (pass)
		pass = reCreateLog();

	    if (pass) 
		return cleanUp();
	    
	    return false;
	} catch (Exception e) {
	    System.err.println(e.getMessage());
	    return false;
	}
	
    }
	

    /**
     * Clear all the hashes we created
     */

    private boolean cleanUp() {
	if (masterhash != null)
	    masterhash.clear();
	if (loghash != null) 
	    loghash.clear();
	return true;
    }

    /**
     * Merge temporary file to the real cache file
     */
    private boolean mergeTempToReal() {

	try {

	    if (Debug.DEBUG_Caching == true) { 
		System.out.println("Merging the temp file into the real .cacheIndexF file"); 
	    } 
 	
	    File tempf = new File(temp);
	    File masterf = new File(master);
	    if (!tempf.exists())
		return false;
	    
	    if (masterf.exists())
		masterf.delete(); //we don't need the master file anymore since the temp file is complete
	    
	    int length = (int) tempf.length();
	    byte[] temp = new byte[length];
	    
	    
	    FileInputStream fin = new FileInputStream(tempf);
	    
	    masterf.createNewFile(); //create a new file
	    FileOutputStream fout = new FileOutputStream(masterf);
	    
	    fin.read(temp);  //copy from one to the other
	    fout.write(temp);
	    
	    //delete the temp file
	    tempf.delete();
	    return true;
	}  catch (IOException e) {
	    System.err.println(e.getMessage());
	    return false;
	}
    }

    /**
     * Write a Date to the log file (we have to account for nulls)
     */

    private boolean writeDate(FileOutputStream ftemp, Date date) {
	try {
	    if (date == null)
		ftemp.write(new String("null" + token_separator).getBytes());
	    else
		ftemp.write(new String(date2Str(date)+ token_separator).getBytes());
	    return true;
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	    return false;
	}
	
    } 
    
    /**
     * Write the temporary cache index file
     */

    private boolean writeTempIndex() {

	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("Writing the temporary index file"); 
 	} 
 	

	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("The number of elements in the Masterash is: " + masterhash.size());
 	}

	if (ran_once) {
	    CacheBuffer cb = CacheBuffer.getInstance();
	    cb.replaceBuffer(masterhash);
	}
	
	ran_once = true;
	
	
	try {
	    FileOutputStream ftemp = new FileOutputStream(temp);
	    
	    for (Enumeration e = masterhash.elements() ; e.hasMoreElements() ;) {
		BufferEntry entry = (BufferEntry) e.nextElement();
		HandleWebPage hp = new HandleWebPage();
		if (hp.FileisInCache(entry.url, false)) {
		    ftemp.write(new String(token_separator + entry.url + token_separator).getBytes());
		    writeDate(ftemp, entry.getExpired());
		    writeDate(ftemp, entry.getModified());
		    writeDate(ftemp, entry.getRetrieved());
		    ftemp.write(new String(entry.getFolder() + token_separator).getBytes());
		    ftemp.write(new String("\n").getBytes());
		}
	    }

	    ftemp.close();
	    //don't need a log file if the temp file exists
	    new File(log).delete();
	    return true;
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	    return false;
	}
	
    }

    /**
     * Activate changes in the log to main memory first
     */
    
    private boolean actChangesInLog() throws Exception{
	if (Debug.DEBUG_Caching == true) { 

	    System.out.println("Activating changes in the log"); 
 	
 	}
	File logf = new File(log);
	if (!logf.exists())
	    return false;
	
	if (logf.canRead()) {
	    InputStream inn = new FileInputStream(log);
	    BufferedReader in = new BufferedReader (new InputStreamReader(inn));
	    String line;
	    while ((line = in.readLine()) != null) {
	        actLogEntry(line);
	    }
	    return true;
	}

	return false;
	
    }

    /**
     * Activate each line in the log and update the main index file
     */

    private boolean actLogEntry(String line) {

	if (Debug.DEBUG_Caching == true) { 
	    
	    System.out.println("^^^^^^^Writing changes from the log to the master hash"); 
 	} 
 	
	try {
	    StringTokenizer st = new StringTokenizer(line, token_separator, false);
	    if (st.countTokens() > 5) {
		String command = st.nextToken().trim();
		if (command.equals("<ADD>")) {
		    if (Debug.DEBUG_Caching == true) {
			System.out.println("In add"); 
		    }
		return addToMasterHash(st);
		}
		
		else if (command.equals("<DELETE>"))
		    return removeFromMasterHash(st);
		else if (command.equals("<UPDATE>"))
		    return updateMasterHash(st);
	    }
	    return false;
	} catch (Exception e) {
	    System.err.println(e.getMessage());
	    return false;
	}
    }
	

    /**
     * removes an entry from the masterhash if the log says so
     */

    private boolean removeFromMasterHash(StringTokenizer st) {
	if (st.countTokens() < 3)
	    return false;

	if (st.nextToken().trim().equals("<BEGIN>")) { //this is where the begin should be
	    String url = st.nextToken();
	    if (st.nextToken().trim().equals("<END>")) { 
	        masterhash.remove(url);
		return true;
	    }
	}
	return false;
    }

	
    private boolean updateMasterHash(StringTokenizer st) throws ParseException{
	if (st.countTokens() < 5)
	    return false;


	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("Updating Master Hash"); 
 	
 	}
	if (st.nextToken().trim().equals("<BEGIN>")) { //this is where the begin should be
	    String url = st.nextToken();
	   
	    BufferEntry entry = (BufferEntry) masterhash.get(url);   
	    if (entry == null) 
		return false;

	    entry.setUrl(url);
	    entry.setExpired(setDate(st.nextToken()));
	    entry.setModified(setDate(st.nextToken()));
	    entry.setRetrieved(setDate(st.nextToken()));
	    entry.setFolder(st.nextToken());
	
       
	    if (st.nextToken().trim().equals("<END>")) {
		//the entry has already been updated in the cachebuffer
		//masterhash.put(entry.getUrl(), entry);
		return true;
	    }
	}
	return false;
	    
    }
    /**
     * add to the masterhash from the log
     */
    private boolean addToMasterHash(StringTokenizer st) throws ParseException {	
	if (st.countTokens() < 5) {
	    System.err.println("The number of tokens for add is: " + st.countTokens() + " -  Needs to be at least 5");
	    return false;
	}
	
	BufferEntry entry = new BufferEntry();

	if (st.nextToken().trim().equals("<BEGIN>")) { //this is where the begin should be
	    if (Debug.DEBUG_Caching == true) { 

		System.out.println("In begin"); 
 	
 	}
	    entry.setUrl(st.nextToken());
	    entry.setExpired(setDate(st.nextToken()));
	    entry.setModified(setDate(st.nextToken()));
	    entry.setRetrieved(setDate(st.nextToken()));
	    entry.setFolder(st.nextToken());
	}


	if (st.nextToken().trim().equals("<END>")) {
	    masterhash.put(entry.url, entry);

	    if (Debug.DEBUG_Caching == true) { 
		System.out.println("The entry just added looks like: " + entry.toString()); 
 	
	    }
	    return true;
	}
    
	return false;
    }


    /**
     * Reads the cacheIndexF file from disk into memory
     */
    public Hashtable readMasterFile() throws IOException, FileNotFoundException, ParseException{
	File masterf = new File(master);
	if (!masterf.exists())
	    return null;
	

	if (Debug.DEBUG_Caching == true) { 
	    System.out.println("Reading master file"); 
 	
 	}
	if (masterf.canRead()) {
	    FileInputStream fmasterinn = new FileInputStream(masterf);
	    BufferedReader fmasterin = new BufferedReader (new InputStreamReader(fmasterinn));
	    String line;
	 
	    while ((line = fmasterin.readLine()) != null) {

		if (Debug.DEBUG_Caching == true) { 
		    System.out.println("The line is: " +  line); 
		} 
 
	        writeToMemFromCacheIndexF(line);
	    }
	    return masterhash;
	}

	return null;
	
    }

    /**
     * set a date in the hash table
     */

    private Date setDate(String date_str) {
	date_str = date_str.trim();
	if (date_str.equals("null")) 
	    return null;

	else
	    try {
       		return DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL).parse(date_str.trim());
	    } catch (ParseException e) {
		System.err.println(e.getMessage());
		return null;
	    }
    }


    /**
     * add to the in-memory hash from the cacheIndexF file
     */
    private boolean writeToMemFromCacheIndexF(String response) throws ParseException{
	StringTokenizer st = new StringTokenizer(response, token_separator);

	if (Debug.DEBUG_Caching == true) { 
 	 System.out.println("In write to mem from Cache: " + st.countTokens()); 
 	} 
 	
	if (st.countTokens() == 5) {
	    BufferEntry entry = new BufferEntry();
	    entry.setUrl(st.nextToken());
	    entry.setExpired(setDate(st.nextToken()));
	    entry.setModified(setDate(st.nextToken()));
	    entry.setRetrieved(setDate(st.nextToken()));
	    entry.setFolder(st.nextToken());

	    if (Debug.DEBUG_Caching == true) { 
		System.out.println("The entry read from the cacheindexF is" + entry); 
	    } 
 	
	    masterhash.put(entry.getUrl(), entry);
	    return true;
	}
       
	return false;
    }

   
    /**
     * Add a message to the log file. <br>
     * for internal use only.
     */
    private boolean addToLogFile(String message) {
	try {
	    flog.write(new String(message).getBytes());
	    return true;
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	    return false;
	}
    }


    /**
     * Add a "delete" message to the log file.
     */ 

    public boolean addDeleteMessToLogFile(String url)  {
	String new_message = token_separator + "<DELETE>" + token_separator + "<BEGIN>"+
	    token_separator + url  + 
	    token_separator + "<END>" + token_separator + "\n";
	
	merged = false;
	return addToLogFile(new_message);
    }
    

    /**
     * Add a "delete" message to the log file.
     */

    public boolean addDeleteMessToLogFile(BufferEntry entry)  {
	String new_message = token_separator + "<DELETE>" + token_separator + "<BEGIN>"+ 
	    token_separator + entry.getUrl() + 
	    token_separator + "<END>" + token_separator + "\n";
	
	merged = false;
	return addToLogFile(new_message);
    }
	
    /**
     * Convert a date into a string
     */

    private String date2Str(Date date) {
	try {
	    String mystring = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL).format(date);
	    return mystring;
	} catch (NullPointerException e) {
	    return "null";
	}
    }

    /**
     * Add an "add" message to the log file.
     */

    public boolean addAddMessToLogFile(BufferEntry entry) {
	String new_message = token_separator + "<ADD>" + token_separator + "<BEGIN>" + token_separator + 
	    entry.getUrl() + token_separator + 
	    date2Str(entry.getExpired()) + token_separator +
	    date2Str(entry.getModified()) + token_separator +
	    date2Str(entry.getRetrieved()) + token_separator +
	    entry.getFolder() + token_separator +
	    "<END>" + token_separator + "\n";

	merged = false;
	return addToLogFile(new_message);
    }

    /**
     * Add an "update" message to the log file
     */

    public boolean addUpdateMessToLogFile(BufferEntry entry) {
	
	String new_message = token_separator + "<UPDATE>" + token_separator + "<BEGIN>" + token_separator + 
	    entry.getUrl() + token_separator + 
	    date2Str(entry.getExpired()) + token_separator +
	    date2Str(entry.getModified()) + token_separator +
	    date2Str(entry.getRetrieved()) + token_separator +
	    entry.getFolder() + token_separator +
	    "<END>" + token_separator + "\n";
	
	merged = false;
	return addToLogFile(new_message);
    }

    /**
     * Re-Create Log file
     */

    private boolean reCreateLog() {
	try {
	    if (flog != null) 
		flog.close();
	
	    File logfile = new File(log);
	    if (logfile.exists())
		logfile.delete();
	    
	    logfile.createNewFile();
	    flog = new FileOutputStream(log);
	    return true;
	}  catch (IOException e) {
	    System.err.println(e.getMessage());
	    return false;
	}
	
    }
}
