package HTTPMethods;
import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/** 
 * this package will contain all the methods that we use on requests that
 * Netscape issues.
 */

import StartAllServers.Debug;
public class ParseRequest {

    public static String convertCachePathToURL(String cachepath)
	{
	    cachepath=cachepath.trim();
	    

	    int len = Caching.Constants.CACHE_DIR.length();
	    
	    if (cachepath.startsWith("/")) {
		String temp = cachepath.substring(len+1, cachepath.length());
		return "http://" + temp;
	    }

	    return "http://" + cachepath;
	}
    
	    
					      
    /**
     * Parse out the port from the request and return it.
     */
    public static String getHostPort (String request) {
	String host = getHostName(request, false);
	int col_pos;
	String port;
	col_pos = host.indexOf(":");
	if (col_pos < 0) 
	    port = "80";
	else
	    port = host.substring(col_pos+1);
	return port;
    }

    /**
     * Just return the hostname. <br>
     * returns the hostname given a URL
     */
    public static String getHostName(String URL) {
	String hostname;
	int begin = URL.indexOf("://") + 3;
	int end;
	if ((end = URL.indexOf(":", begin)) > -1) {
	    hostname = URL.substring(begin, end);
	    return hostname.trim();
	}
	else if ((end = URL.lastIndexOf("/")) > -1) {
if (Debug.DEBUG_HTTPMethods == true) { 
 	 System.out.println("The end is: " + end + " The begin is: " + begin); 
 	}
	    hostname = URL.substring(begin, end);
	    return hostname.trim();
	}
	
	return null;
    }

    /**
     * Parse out the hostname from the request issued to this server. <br>
     * This procedure takes 2 arguments:<br>
     * request - the request being parsed<br>
     * strip_col - whether to strip the colon and port (true = yes)
     */
    public static String getHostName (String request, boolean strip_col) {
	String host;
	int host_beg;
	int host_end;
	int col_pos;
	host_beg = request.indexOf("Host: ");
	host = request.substring(host_beg + 6);
	host_end = host.indexOf("\n");
	host = host.substring(0, host_end);
	if ((col_pos = host.indexOf(":")) > -1 && strip_col) //if there is a port specification
	    return  host.substring(0, col_pos);  //drop it
	else
	    return host;
    }

    /** 
     * Strip the hostname from the URL.
     * Return the URL without the hostname
     */
     public static String stripHostname(String url){
	
	int ind = url.indexOf("//");
	ind = url.indexOf("/", ind+3);
	String ret = url.substring(ind+1);
	return  "http://" + ret;
    }


    /**
     * Parse out the filename from a request made by Netscape
     */
    public static String getFilename (String request) {
	if (request == null)
	    return null;

	StringTokenizer st = new StringTokenizer(request);
	if (st.countTokens() > 2) {
	    st.nextToken();
	    String url = st.nextToken();
	    int slsh_slsh = url.indexOf("//");

	    //if it is an absolute link
	    if (slsh_slsh > -1)
		url = url.substring(slsh_slsh+2);
	
	    //we have a relative link
	    int first_slsh = url.indexOf("/");
	    String filename = url.substring(first_slsh);
	    return filename;
	    
	   
	}
	return null;
    }
	

    /**
     * Parse out the filename and Http Version given a request.<br>
     */
    
    public static String getFilenameHttpVer (String request) {
	int slsh_slsh;
	slsh_slsh = request.indexOf("//");
	int first_slsh;
	int new_line;
	
	//if it is an absolute link
	if (slsh_slsh > -1)
	    request = request.substring(slsh_slsh+2);
	
	//we have a relative link
	first_slsh = request.indexOf("/");
	new_line = request.indexOf("\n");
	String filename = request.substring(first_slsh, new_line);
	return filename;
    }

    /** 
     * Attach index.html to end of path. <br>
     * some servers can't do it themselves
     */
    public static String attachIndexHtml (String path) {
	//int pos_http = path.indexOf(" ");
	//path = path.substring(0, pos_http);
	path = path.trim();
	
	int end = path.lastIndexOf("/");
	String filename = path.substring(end +1, path.length());
	if(filename.length() == 0){
	    filename="index.html";
	    path = path + filename;
	}
	
	return path;
    }

    /**
     * Checks whether a request is destined for the localhost. <br>
     * returns true if it is <br>
     * (1) checks whether the request has lolcahost as the destination <br>
     * (2) checks whether the Domain names of the host in the request and local machine match<br>
     * (3) checks whether the IP addresses of the host in the request and the local machine match
     */
    public static boolean addressedToMyHost(String URL) {	 
	String hostname = getHostName(URL);
if (Debug.DEBUG_HTTPMethods == true) { 
 	 System.out.println("The hostname is: " + hostname); 
 	}
	
	//this would be a weird error in our case
	if (hostname == null) {
	    //better to return true so we don't cache
	    return true;
	}

	try {
	    //check if it is the localhost
	    if (hostname.equals("localhost") ||
		//the name doesn't match
		(InetAddress.getLocalHost().getHostName().trim().equals(hostname)) ||
		//the IP doesn't match
		(InetAddress.getByName(InetAddress.getLocalHost().getHostName().trim()).equals(hostname)))
		return true;
	    
	} catch (UnknownHostException e) {
	    System.err.println(e.getMessage());
	    return true;
	}
	return false;
    }


    /**
     * Remove the referer line from the request.<br>
     */
    public static String remove_referer(String request) {
	StringBuffer temp = new StringBuffer(request);
	int beg = request.indexOf("Referer: ");
	int end = request.indexOf("\n", beg);
	if ( (beg != -1) && (end !=-1) ){
	    temp.delete(beg, end);
	    return temp.toString();
	}
	return request;
    }

    
    /**
     * Determine the type of request that is being issued. <br>
     * see if the request is of type GET, PUT, POST, etc. <br>
     * return the string representing the type of request
     */
    public static String typeOfReq(String request) {
	if (request.indexOf("GET") > -1) 
	    return "GET";
	else if (request.indexOf("PUT") > -1)
	    return "PUT";
	else if (request.indexOf("POST") > -1)
	    return "POST";
	else if (request.indexOf("HEAD") > -1)
	    return "HEAD";
	return "UNKNOWN";
    }
    
    /**
     * Get the URL from the request.<br>
     */
    public static String getURL (String request) {
	int beg = request.indexOf("http://");
	int end = request.indexOf(" ", beg);
	return attachIndexHtml(request.substring(beg, end));
    }
    
    /**
     * redo the GET request. <br>
     * Eliminat the http:// if it appears there <br>
     * Not doing so causes problems for some GET requests
     */
    public static String redoGet (String request) {
	StringBuffer temp = new StringBuffer(request);
	String myURL = getURL(request);
	//since http:// is 7 characters that is our offset
	int end = myURL.indexOf("/", 7);
	String myHost = myURL.substring(7, end);
if (Debug.DEBUG_HTTPMethods == true) { 
 	 System.out.println("My host is: " + myHost); 
 	}
	int beg = request.indexOf("http://");
	end = request.indexOf("/", beg+7);
	temp.delete(beg, end);
	//temp.insert(beg, myPath);
if (Debug.DEBUG_HTTPMethods == true) { 
 	 System.out.println("&&&&&&&&&MY REQUEST IS:\n"+ temp.toString() + "\n"); 
 	}
	return temp.toString();
    }
    
       
    /**
     * make the connection NOT keep-alive. <br>
     * removes the line Proxy-Connection: Keep-Alive from the request
     */
    public static String remove_proxy(String request) {
	StringBuffer temp = new StringBuffer(request);
	int beg = request.indexOf("Proxy-Connection: Keep-Alive\n");
	int end = beg + 29;
	temp.delete(beg, end);
	return temp.toString();
    }
}
