package Misc;
import java.io.*;
import java.lang.*;
import java.util.*;

import StartAllServers.Debug;
public class Battery {
    
    private static Battery myself;

    /**
     * Make an instance of the Buffer. <br>
     */
    private Battery(int threshold) {
	setThreshold(threshold);
    }


    /**
     * Get an instance of this buffer. <br>
     * Since we want this buffer to be global everyone needs to create an instance of this
     * object before making any modifications to any part of it. If the object doesn't
     * exist it will be made anew and a reference to the new instance will be returned
     */
    public static synchronized Battery getInstance(int threshold) {
	if (myself == null) {
	    myself = new Battery(threshold);
	}
	return myself;
    }

    
    /**
     * Get an instance of this buffer. <br>
     * Since we want this buffer to be global everyone needs to create an instance of this
     * object before making any modifications to any part of it. If the object doesn't
     * exist it will be made anew and a reference to the new instance will be returned
     * DEFAULT THRESHOLD: 20%
     */
    public static synchronized Battery getInstance() {
	if (myself == null) {
	    myself = new Battery(20);
	}
	return myself;
    }


    int bufferPercent;
    int threshold;
    
    /**
     * return the last battery level known
     */

    public int getBattLevel() {
	return this.bufferPercent;
    }


    /**
     * return the threshold 
     */

    public int getThreshold() {
	return this.threshold;
    }



    /**
     * Set the battery level
     */
    
    public void setBattLevel(int percent) {
	this.bufferPercent = percent;
    }


    /**
     * Set the threshold
     */

    public void setThreshold(int threshold) {
	this.threshold = threshold;
    }

    /**
     * Print out the threshold and the amount of battery power
     */

    public void printBatteryLevel() {
	    System.out.println("The Battery Level is: " + bufferPercent + "%");
	    System.out.println("The Threshold is: " + threshold + "%");
	
    }
	
    /**
     * Print the current battery level and threshold
     */

    public String toString() {
	return getBattLevel() + "% " + getThreshold() + "%";
    } 

    /**
     * update Battery information by running the apm command
     */

    public void updateBatteryLevel() {
	StringBuffer battery_str = new StringBuffer();

	try {
	    //execute command and set-up streams
	    Runtime Current_RunTime = Runtime.getRuntime();    
	    Process output = Current_RunTime.exec("cat /proc/apm");
	    InputStream in = output.getInputStream();
	    int b;
	    
	    //read from OutPutStream
	    while ((b = in.read()) > 0) {
		battery_str.append((char) b);
	    }
	    
	    //close the stream we read from
	    in.close();
	    
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	}
	
	//proof for all you skeptics
	if (Debug.DEBUG_Misc == true) { 
	    System.out.print("The battery string is: " + battery_str); 
 	}

	StringTokenizer st = new StringTokenizer(battery_str.toString());
	while (st.hasMoreTokens()) {
	    String test = st.nextToken();
	    if (test.indexOf("%") > -1) {
		setBattLevel(new Integer(test.substring(0, test.indexOf("%"))).intValue());
	    }
	}
    }

    public boolean batteryOK() 
	{
	    if (getBattLevel() == -1)
		return true;

	    if (getBattLevel() > getThreshold())
		return true;
	    
	    else return false;
	}
    
}










