package Misc;
import Caching.Constants;
import StartAllServers.Debug;
import StartAllServers.StartAllServers;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.util.Arrays.*;
import java.lang.Class.*;
import java.lang.reflect.Field;

public class ConfigReader {

    String config_file = "ad_hoc.cfg";
    String[] Constants = {"AD_HOC_SYSTEM_PATH", "GLIMPSE_PATH"};
    String[] Network = {"ASK_FOR_REPORTS_PORT", "SEND_REPORTS_ON_PORT", "WebServerPort","PROXY_SERVER_PORT" , "REPORT_REQUEST_GROUP_ADDRESS"};
    String[] Debug = {"DEBUG_Caching", "DEBUG_GUIServer", "DEBUG_Misc", "DEBUG_ProxyServer", "DEBUG_StartAllServers", "DEBUG_UDPMulticast", "DEBUG_UDPUnicast", "DEBUG_WebClient", "DEBUG_WebServer", "DEBUG_HTTPMethods"}; 
    List ConstantsL;
    List NetworkL;
    List DebugL;

    public ConfigReader() {
	initialize();
    }

    public void buildLists() {
	ConstantsL = Arrays.asList(Constants);
	NetworkL = Arrays.asList(Network);
	DebugL = Arrays.asList(Debug);
	
    }

    public void initialize() {
	buildLists();
	File cfg = new File(config_file);
	if (cfg.exists()) {
	    readAndSet(cfg);
	    System.out.println("******Using Settings from the config file provided******");
	}

	else {
	    System.err.println("******No config File found: Using Defaults********");
	}
    }


    public void readAndSet(File cfg) {
	try {
	    InputStream inn = new FileInputStream(cfg);
	    BufferedReader in = new BufferedReader(new InputStreamReader(inn));
	    String line;
	    int line_num=0;
	    
	    while((line = in.readLine()) != null) {
		//ignore comments
		if (line.indexOf("//") < 0) {
		    line_num++;
		    StringTokenizer st = new StringTokenizer(line, "=");
		    if (st.countTokens() > 1)
			rules(st);
		}
	    }
	} catch (FileNotFoundException e) {
	    System.err.println("Config File not Found: " + e.getMessage());
	} catch (IOException e) {
	    System.err.println("Config file can't be read: " + e.getMessage());
	}
    }


    public void rules(StringTokenizer st) {
	String variable = st.nextToken().trim();
	int loc;
	if (ConstantsL.contains(variable)) {
	    setVariable("Caching.Constants", variable, st.nextToken(), true);

	    //set the derived variables also
	    setDerived(variable);

	    return;
	}
	
	else if (NetworkL.contains(variable)) {
	    setVariable("StartAllServers.StartAllServers", variable, st.nextToken(), true);
	    return;
	}
	
	else if (DebugL.contains(variable)) {
	    setVariable("StartAllServers.Debug", variable, st.nextToken(), true);
	    return;
	}
	else {
	    System.err.println("Don't know what this variable is: " + variable);
	}
    }


    public void setDerived(String variable) {
	if (variable.equals("AD_HOC_SYSTEM_PATH")) {
	    setVariable("Caching.Constants", "CACHE_DIR", Caching.Constants.AD_HOC_SYSTEM_PATH + "/.cache", true);
	    setVariable("Caching.Constants", "GLIMPSE_GEN_FILES", Caching.Constants.AD_HOC_SYSTEM_PATH + "/.glimpse_files", true);
	} else {
	    setVariable("Caching.Constants", "GLIMPSE_EXE", Caching.Constants.GLIMPSE_PATH + "/glimpse", true);
	    setVariable("Caching.Constants", "GLIMPSEINDEX", Caching.Constants.GLIMPSE_PATH + "/glimpseindex ", false);
	    setVariable("Caching.Constants", "GLIMPSEINDEX_EXE", Caching.Constants.GLIMPSEINDEX + "-f", true);
	}
    }

    public void setVariable(String classname, String variable, String value, boolean trim_OK) {
	if (trim_OK) 
	    value = value.trim();

	try {
	    Class clsname = Class.forName(classname);
	    Field fld = clsname.getField(variable);
	    //System.out.println(classname + ": " + variable + ": " + fld.getType());
	    if (fld.getType().equals(Boolean.TYPE))
	    	fld.setBoolean(null, new Boolean(value).booleanValue());
	    else if (fld.getType().equals(Class.forName("java.lang.String")))
		fld.set(null, value);
	    else if (fld.getType().equals(Integer.TYPE))
		fld.setInt(null, new Integer(value).intValue());
	    else if (fld.getType().equals(Long.TYPE))
		fld.setLong(null, new Long(value).longValue());
	    else {
		System.err.println("Unrecognized Field name");
	    }
	

	} catch (ClassNotFoundException e) {
	    System.err.println("Class doesn't exist: " + e.getMessage());
	} catch (NoSuchFieldException e) {
	    System.err.println("Field " + variable + " doesn't exist: " + e.getMessage());
	} catch (IllegalAccessException e) {
	    System.err.println(e.getMessage());
	}
    }

    public static void main(String argv[]) {
	ConfigReader cr = new ConfigReader();
    }
}
			
	
	    
