package ProxyServer;
import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import Caching.*;
import GUIServer.*;
import HTTPClient.*;
import HTTPMethods.*;

import StartAllServers.Debug;
class ServeThread extends Thread {
    Socket client_to_proxy = null;
    int option;
    Socket proxy_to_dest = null;
    final boolean error=false;
    final boolean no_error=true;

    //Constructor
    ServeThread(Socket socket, int option) {
	super("ServeThread");  //not needed but good practice
	this.client_to_proxy = socket;
	this.option = option;
    }

    //Now the thread does something
    public void run() {
	boolean steps_complete = false;
	if (Debug.DEBUG_ProxyServer == true) { 
	    System.out.println("Started Thread"); 
 	}
	String request = getReqFromClient(client_to_proxy);
	if (!ParseRequest.addressedToMyHost(request)) {

	    //let's see if the cache can serve this request
	    if ((new HandleWebPage()).FileisInCache(ParseRequest.getURL(request), false) == true) {
		HandleWebPage hp = new HandleWebPage();
		CacheBuffer cb = CacheBuffer.getInstance();
		
		//if the page is in the cache and it has the expired field set 
		//try to get it from the real web server
		if (cb.getExpiration(ParseRequest.getURL(request)) != null && 
		    cb.getExpiration(ParseRequest.getURL(request)).before(new Date()))
		    steps_complete = makeInternetRequest(request);
		
		//if the page couldn't be retrieved from the real web server or there is no expiration date
		//then get the (Maybe) expired page from the cache
		//we decided that it is better to get something rather than nothing
		//Maria's decision
		if (!steps_complete) {
		    
		    try {
			OutputStream outt = client_to_proxy.getOutputStream();
			DataOutputStream os = new DataOutputStream (outt);
			String url = ParseRequest.getURL(request);
			hp.getFromCache(url, os);
			steps_complete = true;
		    } catch (IOException e) {}
		}
	    }

	    //if it isn't in the cache route the request to the internet
	    if (!steps_complete)
		steps_complete = makeInternetRequest(request);
	}
    
	
	//if something went wrong go to the Ad Hoc Network
	if (steps_complete == false)
	    offNetworkAccess(request);
	

	cleanUp(); //close all sockets if fall through
    }


    //this method is responsible for all the internet requests and responses that are sent
    public boolean makeInternetRequest(String request) {

	boolean steps_complete = false;
	//make a special case for GET requests since these
	//are the only requests we're going to consider caching
	if (steps_complete == false && 
	    ParseRequest.typeOfReq(request).equals("GET")) {
	    boolean status = doGetRequest(request);
	    if (status) steps_complete = true;
	}
	
	//let's try to send a request to the internet of type that
	//isn't GET - this is going to be transparent
	//write to input stream - get something from the output stream
	//make sure to not pick up GET requests that failed in the above step
	else if (steps_complete == false && 
		 request != null &&
		 !ParseRequest.typeOfReq(request).equals("GET")) {
	    if(sendReqToDest(request) == no_error) {
		getResponse(request);
		steps_complete = true;
	    }
	}
	return steps_complete;
    }
    

    //this uses the HTTPClient package found in /HTTPClient
    //as was requested by Maria
    public boolean doGetRequest(String request) {
	String URL = ParseRequest.getURL(request);
	String filename = ParseRequest.getFilename(request);
	String hostname = ParseRequest.getHostName(request, true);
	String port = ParseRequest.getHostPort(request);
	
	HTTPConnection con = new HTTPConnection(hostname, Integer.parseInt(port));

	try {
	    OutputStream outt = client_to_proxy.getOutputStream();
	    DataOutputStream os = new DataOutputStream (outt);
	
	    HTTPResponse rsp = con.Get(filename);
	    
	    if (rsp.getStatusCode() >= 400) {
		if (Debug.DEBUG_ProxyServer == true) { 
		    System.out.println("File doesn't exist on: " + hostname + " filename: " + filename); 
		}

		//sometimes error messages are sent so just write them back to the browser
		//don't cache error messages
		byte[] data = rsp.getData();
		os.write(data, 0, data.length);
		os.close();
	    }
	    
	    else {
		
		//quickly write the data back to the browser so the user doesn't wait or notice the delay
		byte[] data = rsp.getData();
		os.write(data, 0, data.length);
		os.close();

		HandleWebPage hp = new HandleWebPage();
		try {
		    hp.cache(rsp.getEffectiveURI().toURL(), rsp, data);
		} catch (Exception e) {
		    System.err.println("Can't get last URI - generating one from request");
		    hp.cache(request, rsp, data);
		}
	    }

	} catch (Exception e) {
	    System.err.println(e.getMessage());
	    return false;
	}
	return true;
    }


    //read the request from the client (Netscape or IE)
    public String getReqFromClient(Socket client_to_proxy) {
	
	InputStream inn;
	OutputStream outt;
	BufferedReader is;
	PrintWriter os;
	String response;
	String message = "";
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Socket No is:" + client_to_proxy); 
 	}
	
	try {
	    
	    //get raw streams
	    inn = client_to_proxy.getInputStream();
	    outt = client_to_proxy.getOutputStream();
	    
	    //turn into BufferedReader streams
	    is = new BufferedReader(new InputStreamReader(inn));
	    os = new PrintWriter (new OutputStreamWriter(outt), true);
	    
	    //check streams
	    if (inn == null || outt == null || is == null || os == null) {
		System.err.println("Failed to open streams");
		return null;
	    }

	    //read request from client
	    while((response = is.readLine()).length() > 0) {
		message = message + response + "\n";
	    } 
	    
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("\nDone Receiving Data from Client"); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("**************The Request was:\n" + message + "\n"); 
 	}

	
	} catch (Exception e) {
	    System.err.print(e.getMessage());
	    return null;
	}

	return message;
	
    }


    /* Send a request to a destination */
    public boolean sendReqToDest (String request) {
	String host = ParseRequest.getHostName(request, true);
	String port = ParseRequest.getHostPort(request);
	String filename = ParseRequest.getFilenameHttpVer(request);
	InetAddress address;
	Socket proxy_dest;
	InputStream inn;
	OutputStream outt;
	BufferedReader is;
	PrintWriter os;
	int port_num = Integer.parseInt(port);

	//confirm
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("The host is: " + host); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("The port number is: " + port_num); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("The filename is: " + filename); 
 	}

	//remove proxy-connection field
	request = ParseRequest.remove_proxy(request);

	try {
	    //resolve name into nanme/IP (Java construct)
	    address = InetAddress.getByName(host);
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("The address is: " + address); 
 	}
	  
	    //create a socket
	    proxy_dest = new Socket(address, port_num);
	  
	    //check pipe
	    if (proxy_dest == null) {
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Sorry, proxy->dest socket can't connect"); 
 	}
		return error;
	    }

	     //get raw streams
	    inn = proxy_dest.getInputStream();
	    outt = proxy_dest.getOutputStream();
	    
	    //turn into BufferedReader and PrintWriter streams
	    is = new BufferedReader(new InputStreamReader(inn));
	    os = new PrintWriter (new OutputStreamWriter(outt), true);
	    
	    //check streams
	    if (inn == null || outt == null || is == null || os == null) {
		System.err.println("Failed to open streams");
		return error;
	    }

	    // relay the request forward 
	    //String myrequest = redoGet(request);
	    System.out.println ("Writing proxy->dest: " + request);
	    os.println (request);
	    
	    proxy_to_dest = proxy_dest;  //set global variable
      

	} catch (UnknownHostException e) {
	    System.err.println(e.getMessage());
	    //cleanUp();
	    return error;
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	    //cleanUp();
	    return error;
	}

	return no_error;
    }

    //just do a simple network test by looking up some names
    //Currently not being used
    public boolean networkTest() {
	try {
	    InetAddress.getByName("www.yahoo.com");
	} catch (UnknownHostException e) {
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Yahoo failed - let's try another one..."); 
 	}
	    try {
		InetAddress.getByName("www.ask.com");
	    } catch (UnknownHostException ef) {
		System.err.println("Ask failed... - we don't have Network Access");
		//make sure to change the mode
		return false;
	    }
	}
	return true;
    }

    //This method is called when we are no longer on the network
    public void offNetworkAccess(String request) {
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("In off network access"); 
 	}

	/*send the same request but modify the host
	  int beg = request.indexOf("Host: ");
	  int end = request.indexOf("\n", beg);
	  StringBuffer temp = new StringBuffer(request);
	  temp.delete(beg, end);
	  temp.insert(beg, "Host: localhost:9009");
	  String ad_hoc_request = temp.toString();
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("proxy_to_dest:" + proxy_to_dest); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("client_to_proxy:" + client_to_proxy); 
 	}
	*/
	if (request != null) { 
	    GuiMain gm = new GuiMain(client_to_proxy);
	    gm.parseAndDecide(request);
	}
    }


    //get a response from the destination host
    public void getResponse(String request) {
	String response;
	OutputStream outt;
	DataOutputStream os;
	String req_type = ParseRequest.typeOfReq(request);

	//confirm
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Waiting for response"); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Client to Proxy: " + client_to_proxy); 
 	}
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("Proxy to Dest: " + proxy_to_dest); 
 	}
	
	try {
	    //get raw output streams
	    outt = client_to_proxy.getOutputStream();
	  
	    //turn into something useful
	    os = new DataOutputStream (outt);

	    String url = ParseRequest.getURL(request);

	    //get the page from the internet
	    getFromInternet(url, os);
	    

	    //close all open streams/sockets
	    cleanUp();

	} catch (Exception e) {
	    System.err.println(e.getMessage());
	}
    }


   //need a synchronized method when trying to decide how to cache files
    public synchronized FileOutputStream setupCacheFile(String url) {
	HandleWebPage hp = new HandleWebPage();
	FileOutputStream fout = hp.saveToCache(url);
	return fout;
    }


    //get from the internet and cache if needed
    public boolean getFromInternet(String url, DataOutputStream os) {
	InputStream inn;
	DataInputStream is;
	
if (Debug.DEBUG_ProxyServer == true) { 
 	 System.out.println("URL IS: " + url); 
 	}

	try {
	    inn = proxy_to_dest.getInputStream();
	    is = new DataInputStream (inn);
	    byte b[] = new byte[1024];
	    int l;
	    
	    //read from destination server and write to client
	    while ((l = is.read(b)) > 0) {
		os.write(b, 0, l);
		b = new byte[1024]; //need to clear out the byte array
	    }
	   
	    return true;
	    
	} catch (IOException e) {
	    System.err.println("IOException has occurred: " + e.getMessage());
	    return false;
	}
    }
	
    

    //close all sockets that are still open
    public void cleanUp() {
	System.err.println("Closing All sockets");
	try {
	    if (client_to_proxy != null)
		client_to_proxy.close();
	    if (proxy_to_dest != null)
		proxy_to_dest.close();
	} catch (IOException e) {
	    System.err.println(e.getMessage());
	}
    }
}
