package StartAllServers;
import java.io.*;
import UDPMulticast.*;
import UDPUnicast.*;
import WebServer.*;
import ProxyServer.*;
import Caching.*;
import GUIServer.*;
import Misc.*;

//starts all the servers...

public class StartAllServers {
    public static int ASK_FOR_REPORTS_PORT = 9001;
    public static int SEND_REPORTS_ON_PORT = 9002;
    public static int WebServerPort = 9000;  //needs to be set in SimpleServer.prefs
    public static String REPORT_REQUEST_GROUP_ADDRESS = "230.0.0.1";
    public static int PROXY_SERVER_PORT = 8080;
    public static String AD_HOC_IP_ADDRESS = "128.59.15.50";
    

    public StartAllServers() {

	//read the config file
	new ConfigReader();

	//sets the Ad Hoc Ip Address
	new AssignAdHocIPAddress("eth0");

	//set the Multicast Address in the routing table
	new CreateMulticastRoute("eth0");
	 
	//start the battery Monitor utility
	//check the battery level every hour
	new BatteryChecker(360000).start();

	//this will get the cacheIndexFile merged with any log that
	//is lying around.
	Logging log = Logging.getInstance();

	//startup the Log Manager
	//this will merge the log with the original periodically
	//every minute - probably not needed more often than that
	new LogManager(120000).start();

	//start up our GlimpseManager
	//periodically run the glimpse thread and re-index everything in the
	//cache
	//run glimpse to re-index the entire cache directory
	new GlimpseManager(40000).start();

	//create the buffer which will keep track of the cache
	CacheBuffer myCacheBuffer = CacheBuffer.getInstance();

	//start up our CacheManager
	//update the cache every 2 minutes
	//delete stuff that is more than 60 minutes old
        new CacheManager(120000, 60).start();
	
	//create the buffer which will keep track of the queries
	QueryReportBuffer mybuffer = QueryReportBuffer.getInstance();

	//start running the Selection Function
	new Selection().start();
	
	//create a UDP Multicast Receiver
	new MulticastReceiveThread(ASK_FOR_REPORTS_PORT).start();

	//create a UDP Unicast Receiver
	new UDPUnicastReceiverHelper(SEND_REPORTS_ON_PORT).start();

	//start the Proxy Server
	new startProxy().start();

	//start our local cacheServer
	//SimpleServer myWebServer = new SimpleServer();
	//SimpleServer.startWebServer();
	new cacheServer().start();


    }


    public static void main(String [] args) {

	new StartAllServers();

    }
}
