package UDPMulticast;
import java.io.*;
import java.net.*;
import java.util.*;
import StartAllServers.Debug;
import StartAllServers.*;

public class MulticastSendThread extends Thread {

    DatagramSocket socket;
    String keywords;
    int num_times_to_send = -1;
    long until = -1;
    int datagram_port;

    public MulticastSendThread(int listen_port, String keyword, int ntts) {
	super("MulticastSendThread");
	int datagram_port = generate_port();
	this.keywords = keyword;
	this.num_times_to_send = ntts;
    }

    public MulticastSendThread(int listen_port, String keyword, long until) {
	super("MulticastSendThread");
	int datagram_port = generate_port();
	this.keywords = keyword;
	this.until = until;
    }

    synchronized public int generate_port() {
	int port = 0;
	while (port == 0)
	    try {
		port = 5000 + (int) (Math.random() * 1000); 
		socket = new DatagramSocket(port);
	    } catch (Exception e) {
if (Debug.DEBUG_UDPMulticast == true) { 
 	 System.out.println(e.getMessage()); 
 	}
		port = 0;
		continue;
	    }
	System.err.println("Using Datagram Port: " + port);
	return port;
    }

    //send UDP requests but don't close sockets for efficiency sake and I don't
    //want to keep looking for new free ports.
    public void sendUDPRequest() {
	  try {
                byte[] buf = new byte[1024];
		
                String dString = null;
		dString = "<Query>" + keywords + "</Query>";
                buf = dString.getBytes();
if (Debug.DEBUG_UDPMulticast == true) { 
 	 System.out.println("MULTICAST_SEND_THREAD:" + keywords); 
 	}
		
		// send it
                InetAddress group = InetAddress.getByName("230.0.0.1");
	
		//packet must originate from some other port
		//since the multicaster is already using a port
		//CHANGE THIS IF MULTICAST PORT HAS CHANGED
                DatagramPacket packet = new DatagramPacket(buf, buf.length, group, StartAllServers.ASK_FOR_REPORTS_PORT);
                socket.send(packet);
		
		try {
		    //sleep for 10 seconds
		    Thread.sleep(10000);
		} catch (InterruptedException e) {
if (Debug.DEBUG_UDPMulticast == true) { 
 	 System.out.println("Error with sleep: " + e.getMessage()); 
 	}
		}
		
            } catch (IOException e) {
                e.printStackTrace();
            }
    }
	    

    public void run() {

	//check which condition we're enforcing
	if (until == -1 && num_times_to_send > -1)
	    while (num_times_to_send > 0) {
		sendUDPRequest();
		num_times_to_send--;
	    }
	
	else if (until > -1 && num_times_to_send == -1) {
	    while (System.currentTimeMillis() <  until)
		sendUDPRequest();
	}
	
	//else do nothing because something went wrong
	else {} 
          
	//close the socket we used to send UDP messages over
	socket.close();
	
    }
}
